@@ -1,6 +1,6 @@
 #include <stdio.h>
 
-void vcl_stuff()
+void vcl_stuff(void)
 {
   printf("Placeholder for a file with strange name\n");
 }
